/*
 REACTIVE AUTONOMOUS BLACKHOLE LIST SERVER
 COPYRIGHT (C) 2005 DEEP LOGIC, INC.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _HASH_H
#define _HASH_H

enum { ht_num_primes = 28 };

/* hash root */
struct hash {
    long size;
    long items;
    struct ht_node **tbl;
};

/* hash node */
struct ht_node {
    char *key;
    void *ptr;
    struct ht_node *next;
};

/* cursor for iterative searches */
struct ht_c {
  long iter_index;
  struct ht_node *iter_next;
};

/* constructor and destructor */

struct hash *	ht_create(long size);
int		ht_destroy(struct hash *ht);

/* public functions */

void *		ht_ptr(struct hash *ht, const char *key);		/* Retrieve value */
struct ht_node *ht_find(struct hash *ht, const char *key);		/* Retrieve node */
struct ht_node *c_ht_first(struct hash *ht, struct ht_c *c);		/* Set cursor at first node */
struct ht_node *c_ht_next(struct hash *ht, struct ht_c *c);		/* Step cursor to next node */
int 		ht_delete(struct hash *ht, const char *key);		/* Delete a node */
int 		ht_set(struct hash *ht, const char *key, void *val);	/* Set Pointer */



/* private functions */

long		ht_hashcode(struct hash *ht, const char *key);	
struct ht_node *ht_node_create(const char *key);

#endif /* _HASH_H */

