/*
 REACTIVE AUTONOMOUS BLACKHOLE LIST SERVER
 COPYRIGHT (C) 2005 DEEP LOGIC, INC.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifdef HAVE_CONFIG_H
#include <auto-config.h>
#endif

#ifdef TIME_WITH_SYS_TIME
#   include <sys/time.h>
#   include <time.h>
#else
#   ifdef HAVE_SYS_TIME_H
#       include <sys/time.h>
#   else
#       include <time.h>
#   endif
#endif

#ifndef _RABL_CLIENT_H
#  define _RABL_CLIENT_H

#define	SYNTAX	"Syntax: rabl_client %s [-cs] [address] [-d] [directory]\n"
#define PROTOCOL_VERSION	"1"	/* Protocol Version 1 */
#define SH_PACKET_MAX	1024

int authenticated;
int sockfd;

#if ((defined(__sun__) && defined(__svr4__)) || (defined(__sun) && defined(__SUNPRO_C))) && !defined(u_int32_t) && !defined(__BIT_TYPES_DEFINED__)
#define __BIT_TYPES_DEFINED__
typedef unsigned int u_int32_t;
typedef unsigned short u_int16_t;
typedef unsigned char u_int8_t;
#endif

struct rabl_client_entry {
  time_t last_hit;
};

struct rabl_data_ipv4 {
  char type;
  u_int32_t address;
};

struct rabl_data_ipv6 {
  char type;
  char address[39];
};

struct rabl_data_auth {
  u_int32_t uid;
};

/* Functions */
int	destroy_config(void);
int	process_function(int func, char *address);
int	establish_connection(void);
int	query_ipv4_address(int sockfd, char *address);
int	query_ipv6_address(int sockfd, char *address);
int	post_ipv4_address(int sockfd, char *address);
int	post_ipv6_address(int sockfd, char *address);
int	authenticate(int sockfd);
int	watch_directory(const char *dir);

#define DATA_QUERY		0x00
#define DATA_POST_SPAM		0x01
#define DATA_POST_HAM		0x02
#define RABL_LOGIN		0xFE
#define RABL_LOGOUT		0xFF

#define PT_IPV4	0x01
#define PT_IPV6 0x02
#define	PT_AUTH	0x03

#define FUNC_QUERY	0x01
#define FUNC_POST_SPAM	0x02
#define FUNC_WATCH_DIR	0x03

#ifndef EINVAL
#       define EINVAL                   -0x01
  /* Invalid call or parms - Already Initialized or shutdown */
#endif
#define EUNKNOWN                -0x02   /* Unknown/unexpected error */
#define EFILE                   -0x03   /* File error */
#define ELOCK                   -0x04   /* Lock error */
#define EFAILURE                -0x05   /* Failed to perform operation */

#endif /* _RABL_CLIENT_H */

